package org.planet.snap;

import dermi.*;

import java.io.*;
import java.util.*;
import java.net.*;

import javax.servlet.http.*;
import javax.servlet.*;
import javax.naming.InitialContext;
import javax.naming.NameNotFoundException;

import org.planet.snap.ds.SnapDataSource;

public class SnapAppStartup extends HttpServlet {
  Application snapApp = null;
  String realPath;

  /*
   * Read the list of components of this application.
   * Parses the snap-war.xml file.
   */
  public ArrayList readComponents() {
    try{
      ArrayList components = new ArrayList();
	  File f = new File(realPath+"/meta-inf/snap-war.xml");
	  if(f.exists()){
	    FileInputStream fi = new FileInputStream(f);
        Properties appProperties = new Properties();
        appProperties.loadFromXML(fi);
 	 
   	    int i = 1;
        while (appProperties.getProperty (Context.SNAP_P2PCOMP + i) != null) {
   	      components.add(realPath + "/WEB-INF/lib/" + appProperties.getProperty(Context.SNAP_P2PCOMP + i));
   	      i++;
   	    }
   	    fi.close();
   	    return components;
	  }
    }catch(Exception e){
      System.out.println("Error while loading components from snap-war.xml");
	  e.printStackTrace();
    }
	return null;
  }
  
  public void init (ServletConfig config) throws ServletException {
    super.init (config);
    ServletContext ctx = config.getServletContext();
	realPath = ctx.getRealPath (File.separator); 

    try {
        // ----- ATTEMPT TO LOAD LOCAL HOSTNAME -----
        String serverName = InetAddress.getLocalHost().getHostName ();
        ctx.setAttribute ("serverName", serverName);
        InitialContext context = new InitialContext();

		// Get the parent ClassLoader. The Jetty classloader.
        Thread current =Thread.currentThread();
		URLClassLoader ucl = (URLClassLoader) current.getContextClassLoader().getParent();
        
		// Add components to Snap classpath
		System.out.println("Loading components to classpath ...");
        ArrayList components = readComponents();
        if(components!=null) {
          Iterator it = components.iterator();
          if(it!=null)
            while(it.hasNext()) {
              String comp_jar = (String)it.next();
              ClasspathHacker.addFile(comp_jar, ucl);
            }
	    }
		
        Properties p = Application.getSnapAppConfig (ctx.getRealPath (File.separator));

        // Create main SNAP application handler
        snapApp = new Application (serverName, p);
        ctx.setAttribute ("snapApp", snapApp);
		
        // Update datasource configuration (if application requires DB persistence)
        if ((p.getProperty (Context.SNAP_PERSISTENCE) != null) &&
            (p.getProperty (Context.SNAP_PERSISTENCE).equals (Context.DATABASE))) {
          try {
            SnapDataSource ds = (SnapDataSource) context.lookup ("java:comp/env/jdbc/SnapDS");
            ds.setApplication (snapApp);
          } catch (NameNotFoundException ne) {
            System.err.println ("Datasource utilization is disabled. If you want to enable DataSources in your webapp, please update your 'web.xml' file.");
          }
        }

        // Rebind application reference
        context.rebind (snapApp.getP2PUrl().replace ('/', '_'), snapApp.getAppLocalNodeHandle());
        System.out.println ("Bound " + snapApp.getP2PUrl().replace ('/', '_') + " to " + snapApp.getAppLocalNodeHandle());

    } catch (Exception e) {
      System.out.println ("------------------------------ EXCEPTION --------------------------------");
      System.out.println ("------------------------ Initializing SNAP App --------------------------");
      System.out.println ("-------------------------------------------------------------------------");
      e.printStackTrace();
      throw new ServletException (e.getMessage());
    }
  }

  public void destroy() {
    super.destroy();

    if (snapApp != null) {
      try {
        Registry.leave ();
        snapApp.close();
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }
}
