package org.planet.snap.deployer;

import java.io.File;
import java.util.Iterator;
import java.util.Properties;
import java.util.StringTokenizer;

import dermi.Registry;
import dermi.exception.RemoteException;
import dermi.exception.NotBoundException;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import org.planet.snap.Context;

public class ListAndRemoveDialog extends TitleAreaDialog {
  private Button back;
  private Button remove;
  private SnapDeployer win;
  private boolean canRemove;
  private Tree t;

  public ListAndRemoveDialog (Shell parentShell, SnapDeployer wd, boolean canRem) {
    super (parentShell);
    win = wd;
    canRemove = canRem;
  }

  protected Control createDialogArea (Composite parent) {
    Composite area = (Composite) super.createDialogArea(parent);
    Composite container = new Composite(area, SWT.NONE);
    container.setLayoutData(new GridData(GridData.FILL_BOTH));

    super.getShell().setText ("Snap Deployer");
    Image img = new Image (super.getShell().getDisplay(), "./images/snap_installer.jpg");
    super.getShell().setImage (img);

    setTitle ("SNAP Web Application Signer and Deployer Wizard");
    setMessage ("The list of available deployed Snap applications is shown below.");
    setTitleImage (img);

    final Label label = new Label(container, SWT.NONE);
    label.setBounds (15, 17, 450, 18);

    if (!canRemove) {
      label.setText ("Please choose a Snap web application to see its details:");
    }
    else {
      label.setText ("Please choose the Snap web application to be removed:");
    }

    t = new Tree (container, SWT.SINGLE | SWT.BORDER);
    TreeItem root = new TreeItem (t, SWT.NONE);
    root.setExpanded (true);

    // Load available list of applications
    addItems (root);

    t.setBounds (15, 40, 460, 80);

    final Label label2 = new Label (container, SWT.NONE);
    label2.setBounds (15, 125, 450, 18);

    final Label label3 = new Label (container, SWT.NONE);
    label3.setBounds (15, 150, 450, 18);

    final Label label_4 = new Label (container, SWT.SEPARATOR | SWT.HORIZONTAL);
    label_4.setBounds (7, 200, 485, 5);

    t.addSelectionListener (new SelectionAdapter() {
      public void widgetSelected (SelectionEvent event) {
        String itemSelected = (t.getSelection())[0].getText();
        System.out.println("WIDGET SELECTED !!  "+itemSelected);
        if (!itemSelected.toLowerCase().startsWith ("snap root")) {
          Properties appProps = null;
          try {
            StringTokenizer tokens = new StringTokenizer (itemSelected, "/");
            tokens.nextToken();   // Skip 'p2p://'
            appProps = (Properties) Registry.lookupSer ("p2p://snap.deim.urv.net/" + tokens.nextToken() + "_METADATA");
          } catch (RemoteException re) {
            re.printStackTrace();
          }
          label2.setText ("App name:\t " + appProps.getProperty (Context.SNAP_APPCONTEXT));
          label3.setText ("App description:\t " + appProps.getProperty (Context.SNAP_APPNAME));
          if (canRemove) {
            remove.setEnabled (true);
          }
        }
        else {
          if (canRemove) {
            remove.setEnabled (false);
          }
          label2.setText ("App name:\t None selected");
          label3.setText ("App description:\t None selected");
        }
      }
    });

    return area;
  }

  private void addItems (TreeItem root) {
    try {
      Iterator<String> appz = win.getSnapAppsFromRegistry().iterator();
      root.setText ("Snap Root");

      while (appz.hasNext()) {
        TreeItem ti = new TreeItem (root, SWT.NONE);
        StringTokenizer tokens = new StringTokenizer (appz.next(), "/");
        String item = tokens.nextToken() + "//";  // 'p2p:'
        tokens.nextToken(); // skip 'snap.deim.urv.net'
        item += tokens.nextToken();
        ti.setText (item);
      }
    } catch (NotBoundException nbe) {
      root.setText ("Snap Root not bound!");
    }
  }

  protected void createButtonsForButtonBar(Composite parent) {
    if (canRemove) {
      createButton (parent, IDialogConstants.CLIENT_ID, "Remove", false);
      remove = getButton (IDialogConstants.CLIENT_ID);

      remove.addSelectionListener (new SelectionAdapter() {
        public void widgetSelected (SelectionEvent event) {
          String itemSelected = (t.getSelection())[0].getText();
          if (!itemSelected.startsWith ("Snap root")) {
            try {
              StringTokenizer tokens = new StringTokenizer (itemSelected, "/");
              tokens.nextToken();   // Skip 'p2p://'
              String elem = tokens.nextToken();
              Registry.unbind ("p2p://snap.deim.urv.net/" + elem + "_METADATA");
              Registry.unbind ("p2p://snap.deim.urv.net/" + elem + "_WARFILE");
              win.shutdownDatabases ("p2p://snap.deim.urv.net/" + elem);
              Registry.unbind ("org.hsqldb.replication.SQLReplicatorFactory$p2p://snap.deim.urv.net/" + elem + "$Persistence");
              t.removeAll();
              TreeItem root = new TreeItem (t, SWT.NONE);
              root.setExpanded (true);
              addItems (root);
            } catch (RemoteException re) {
              re.printStackTrace();
            }
          }
        }
      });
    }

    createButton (parent, IDialogConstants.BACK_ID, IDialogConstants.BACK_LABEL, true);
    createButton (parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);

    Button cancel = getButton (IDialogConstants.CANCEL_ID);
    cancel.addSelectionListener (new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        close();
      }
    });

    back = getButton (IDialogConstants.BACK_ID);
    back.setEnabled (true);
    back.addSelectionListener (new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        Point p = getShell().getLocation();
        close();
        win.openMainOptionsDialog (p);
      }
    });
  }

  protected Point getInitialSize() {
    return new Point(500, 375);
  }
}
